﻿Function Get-NTFSPermission
{
  param
  (
    [Parameter(ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true,Mandatory=$true)]
    [String[]]
    # ...und auch in Objekten, die die Eigenschaft "FullName" oder "Path" besitzen
    # (also zum Beispiel Ergebnisse von Get-ChildItem)
    [Alias('FullName')]
    $Path
  )

  # der process-Block wird für jedes empfangene Pipeline-Element wiederholt:

  Process
  {
    # falls mehrere Pfade kommasepariert angegeben wurden, einzeln bearbeiten:
    $Path | ForEach-Object {
      # Pfad merken für später
      $currentPath = $_
      # NTFS-Berechtigungen lesen
      # das kann zu terminierenden Fehlern führen (bei fehlenden Berechtigungen)
      # daher ist dieser Teil in einen try-Block gestellt (siehe "Fehlerhandler")
      try
      {
        $ACL = Get-Acl -Path $currentPath

        # Access Control Entries einzeln bearbeiten und für jeden davon ein
        # Ergebnisobjekt liefern:
        $ACL.Access | ForEach-Object {
          # Ergebnisobjekt setzt sich zusammen aus VORHANDENEN und NEUEN Eigenschaften
          # Neue Eigenschaften sind "Path", "Identity", "Right" und "Type", denn diese waren
          # vorher nicht vorhanden. Alle bestehenden Eigenschaften, die "Inheri" enthalten,
          # werden übernommen:
          $Result = $_ | Select-Object -Property Path, Identity, Right, Type,  *Inheri*
          # "Path" wird der aktuelle Pfad zugewiesen. Diese Information fehlte in ACL-Objekten bisher:
          $Result.Path = $currentPath
          # den übrigen neuen Eigenschaften werden die unveränderten alten Eigenschaften zugewiesen.
          # dies geschieht nur, um den Eigenschaften bessere (schönere, verständlicherer) Namen zu geben:
          $Result.Identity = $_.IdentityReference
          $Result.Type = $_.AccessControlType
          $Result.Right = $_.FileSystemRights
          # danach wird das Ergebnisobjekt zurückgeliefert:
          $Result
        }
      }
      catch
      {
        Write-Warning "Kein Zugriff auf $currentPath. Fehler: '$_'"
      }
    }
  }
}

# Alle NTFS-Rechte aller Dateien und Ordner im Windows-Ordner
Get-ChildItem -Path $env:windir |
Get-NTFSPermission |
Out-GridView


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
