﻿<#
  .SYNOPSIS
  Searches assemblies for enumeration data types that can be used to declare PowerShell function parameters
  .DESCRIPTION
  By default, only default assemblies are searched that are generally available in PowerShell. 
  With the switch parameter -All, all currently loaded assemblies are searched.
  The results may then include types that are not commonly available and may require modules or assemblies to be loaded first.
  .EXAMPLE
  Find-Enum
  lists all enumeration data types for all default assemblies
  .EXAMPLE
  Find-Enum *comput*
  lists all enumeration data types for all default assemblies that contain at least one value that matches
"*comput*
  .EXAMPLE
  Find-Enum red -All
  finds all enumeration data types in all currently loaded assemblies that contain the value "red"
#>
function Find-Enum
{
    param
    (
        $Value = '*',
        $Name = '*',
        [Switch]
        $All
    )

    $default = 'CommonLanguageRuntimeLibrary', 'Microsoft.CSharp.dll',
      'Microsoft.Management.Infrastructure.dll', 'Microsoft.PowerShell.Commands.Management.dll', 
      'Microsoft.PowerShell.Commands.Utility.dll', 'System.Configuration.dll',
      'System.Configuration.Install.dll', 'System.Core.dll', 'System.Data.dll',
      'System.DirectoryServices.dll', 'System.dll', 'System.Management.Automation.dll',
      'System.Management.dll', 'System.Transactions.dll', 'System.Xml.dll'

[AppDomain]::CurrentDomain.GetAssemblies() | 
  Where-Object { $All -or ($default -contains $_.ManifestModule) } |
  ForEach-Object { try { $_.GetExportedTypes() } catch {} } | 
  Where-Object { $_.isEnum } |
  Where-Object { $_.Name -like $Name } |
  Sort-Object -Property Name |
  ForEach-Object {    
    $rv = $_ | Select-Object -Property Name, Values, Source
    $rv.Name = '[{0}]' -f $_.FullName
    $rv.Source = $_.Module.ScopeName
    $rv.Values = [System.Enum]::GetNames($_) -join ', '
    $rv
  } |
  Where-Object { @($_.Values -split ', ') -like $Value } 
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
