Function New-SelfsignCertificate
{
  [CmdletBinding()]

  param
  (
    [string][Parameter(Mandatory=$true)]$Name,

    [switch]$CreateTrust,

    [ValidateSet('ServerAuthentication','ClientAuthentication','CodeSigning','Email','Timestamp','OCSPSign')]
    $Purpose = 'CodeSigning',

    [datetime]$NotBefore = (Get-Date).AddDays(-1),

    [datetime]$NotAfter = $NotBefore.AddDays(365),

    [int][ValidateSet('1024', '2048')]$KeyLength = 1024,

    [switch]$PrivateKeyExportable
  )

  # Untersttzt die Windows-Version das Anlegen von Zertifikaten?
  $OSversion = ([System.Version]((Get-WmiObject Win32_OperatingSystem).Version)).Major

  if ($OSversion -lt 6)
  {
    Write-Warning 'With this version of windows, you cannot create certificates.'
    return
  }

  # Hashtable mit den Codes der Verwendungszwecke:
  $purposeCode = @{
    ServerAuthentication='1.3.6.1.5.5.7.3.1'
    ClientAuthentication='1.3.6.1.5.5.7.3.2'
    CodeSigning='1.3.6.1.5.5.7.3.3'
    Email='1.3.6.1.5.5.7.3.4'
    Timestamp='1.3.6.1.5.5.7.3.8'
    OCSPSign='1.3.6.1.5.5.7.3.9'
  }

  # DistinguishedName fr Zertifikat herstellen:
  $subject = "CN=$name"
  $SubjectDN = New-Object -ComObject X509Enrollment.CX500DistinguishedName
  $SubjectDN.Encode($Subject, 0x0)

  # Verwendungszweck hinterlegen:
  $OID = New-Object -ComObject X509Enrollment.CObjectID
  $OID.InitializeFromValue($purposeCode.$purpose)
  $OIDs = New-Object -ComObject X509Enrollment.CObjectIDs
  $OIDs.Add($OID)

  # Privaten Schlssel erzeugen:
  $EKU = New-Object -ComObject X509Enrollment.CX509ExtensionEnhancedKeyUsage
  $EKU.InitializeEncode($OIDs)
  $PrivateKey = New-Object -ComObject X509Enrollment.CX509PrivateKey
  $PrivateKey.ProviderName = 'Microsoft Base Cryptographic Provider v1.0'
  $PrivateKey.KeySpec = 0x2
  $PrivateKey.Length = $KeyLength
  $PrivateKey.MachineContext = 0x0

  if ($PrivateKeyExportable)
  {
    $PrivateKey.ExportPolicy = 0x1
  }

  $PrivateKey.Create()

  # Zertifikatanforderung herstellen:
  $Cert = New-Object -ComObject X509Enrollment.CX509CertificateRequestCertificate
  $Cert.InitializeFromPrivateKey(0x1,$PrivateKey,'')
  $Cert.Subject = $SubjectDN
  $Cert.Issuer = $Cert.Subject
  $Cert.NotBefore = $NotBefore
  $Cert.NotAfter = $NotAfter
  $Cert.X509Extensions.Add($EKU)
  $Cert.Encode()

  # Zertifikat gem Request herstellen und installieren
  $Request = New-Object -ComObject X509Enrollment.CX509enrollment
  $Request.InitializeFromRequest($Cert)
  $endCert = $Request.CreateRequest(0x1)
  $Request.InstallResponse(0x2,$endCert,0x1,'')

  # Vertrauen auf lokalem Computer herstellen, indem Zertifikat
  # als Stammzertifizierungsstelle und vertrauenswrdiger
  # Herausgeber eingetragen wird:

  if ($CreateTrust)
  {
    [Byte[]]$bytes = [System.Convert]::FromBase64String($endCert)

    ForEach ($Container in 'Root', 'TrustedPublisher')
    {
      $x509store = New-Object Security.Cryptography.X509Certificates.X509Store $Container, 'CurrentUser'
      $x509store.Open([Security.Cryptography.X509Certificates.OpenFlags]::ReadWrite)
      $x509store.Add([Security.Cryptography.X509Certificates.X509Certificate2]$bytes)
      $x509store.Close()
    }
  }
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erlutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollstndig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag bernehmen eine Haftung fr Folgen, die durch die Ausfhrung
# dieses Skripts entstehen.
#
################################################################
