﻿$code = @'
using System;
using System.Collections.Generic;
using System.Text;
using System.Management.Automation;
using System.Management.Automation.Runspaces;

namespace InProcess
{
    public class InMemoryJob : System.Management.Automation.Job
    {
        public InMemoryJob(ScriptBlock scriptBlock, string name)
        {
            _powerShell = PowerShell.Create().AddScript(scriptBlock.ToString());
            SetUpStreams(name);
        }

        public InMemoryJob(PowerShell powerShell, string name)
        {
            _powerShell = powerShell;
            SetUpStreams(name);
        }

        private void SetUpStreams(string name)
        {
            _powerShell.Streams.Verbose = this.Verbose;
            _powerShell.Streams.Error = this.Error;
            _powerShell.Streams.Debug = this.Debug;
            _powerShell.Streams.Warning = this.Warning;
            _powerShell.Runspace.AvailabilityChanged += new EventHandler<RunspaceAvailabilityEventArgs>(Runspace_AvailabilityChanged);

            int id = System.Threading.Interlocked.Add(ref InMemoryJobNumber, 1);
            if (!string.IsNullOrEmpty(name))
            {
                this.Name = name;
            }
            else
            {
                this.Name = "InProcessJob" + id;
            }
        }

        void Runspace_AvailabilityChanged(object sender, RunspaceAvailabilityEventArgs e)
        {
            if (e.RunspaceAvailability == RunspaceAvailability.Available)
            {
                this.SetJobState(JobState.Completed);
            }
        }

        PowerShell _powerShell;
        static int InMemoryJobNumber = 0;

        public override bool HasMoreData
        {
            get {
                return (Output.Count > 0);
            }
        }
        public override string Location
        {
            get { return "In Process"; }
        }

        public override string StatusMessage
        {
            get { return "A new status message"; }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (!isDisposed)
                {
                    isDisposed = true;
                    try
                    {
                        if (!IsFinishedState(JobStateInfo.State))
                        {
                            StopJob();
                        }

                        foreach (Job job in ChildJobs)
                        {
                            job.Dispose();
                        }
                    }
                    finally
                    {
                        base.Dispose(disposing);
                    }
                }
            }
        }

        private bool isDisposed = false;

        internal bool IsFinishedState(JobState state)
        {
            return (state == JobState.Completed || state == JobState.Failed || state == JobState.Stopped);
        }

        public override void StopJob()
        {
            _powerShell.Stop();
            _powerShell.EndInvoke(_asyncResult);
            SetJobState(JobState.Stopped);
        }

        public void Start()
        {
            _asyncResult = _powerShell.BeginInvoke<PSObject, PSObject>(null, Output);
            SetJobState(JobState.Running);
        }
        IAsyncResult _asyncResult;

        public void WaitJob()
        {
            _asyncResult.AsyncWaitHandle.WaitOne();
        }

        public void WaitJob(TimeSpan timeout)
        {
            _asyncResult.AsyncWaitHandle.WaitOne(timeout);
        }
    }

}
'@

Add-Type -TypeDefinition $code

Function Start-JobInProcess
{
  [CmdletBinding()]
  param
  (
    [ScriptBlock] $ScriptBlock,
    $ArgumentList,
    [string] $Name
  )

  Function Get-JobRepository
  {
    [cmdletbinding()]
    param()
    $pscmdlet.JobRepository
  }

  Function Add-Job
  {
    [cmdletbinding()]
    param
    (
      $job
    )

    $pscmdlet.JobRepository.Add($job)
  }

  if ($ArgumentList)
  {
  $powershell = [PowerShell]::Create().AddScript($ScriptBlock).AddArgument($argumentlist)
  $MemoryJob = New-Object InProcess.InMemoryJob $powershell, $Name
  }
  else
  {
  $MemoryJob = New-Object InProcess.InMemoryJob $ScriptBlock, $Name
  }
  $MemoryJob.Start()
  Add-Job $MemoryJob
  $MemoryJob
}

######################TEST#########################

Write-Warning 'Erhebe Beispiel-Daten. Das kann einige Minuten dauern...'
$data = Get-ChildItem $env:windir -Filter *.dll -Recurse -ErrorAction SilentlyContinue

# dieser Code wird von beiden Hintergrundjobs ausgeführt.
# er empfängt große Datenmengen und gibt diese wieder in zufälliger Reihenfolge zurück
# (Provokationstest):
$code = {
  Get-Random -InputObject $args -Count $args.Count
}

Write-Warning 'Messe klassischen Hintergrund-Job...'

$dauer = Measure-Command {
  $job = Start-Job -ScriptBlock $code -ArgumentList $data
  Wait-Job $job
  $result = Receive-Job $job
  Remove-Job $job
}

'Klassischer Job: {0:N0} ms' -f $dauer.TotalMilliseconds
'Anzahl Ergebnisse: {0:N0}' -f $result.Count
'Datentypen:'
$result |
Get-Member |
Sort-Object -Property TypeName -Unique |
Select-Object -ExpandProperty TypeName

Write-Warning 'Messe InProcess-Job...'

$dauer = Measure-Command {
  $job = Start-JobInProcess -ScriptBlock $code -ArgumentList $data
  Wait-Job $job
  $result = Receive-Job $job
  Remove-Job $job
}

'InProcess-Job: {0:N0} ms' -f $dauer.TotalMilliseconds
'Anzahl Ergebnisse: {0:N0}' -f $result.Count
'Datentypen:'
$result |
Get-Member |
Sort-Object -Property TypeName -Unique |
Select-Object -ExpandProperty TypeName


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
